/* Apply function to gmsh mesh */
// gcc function.c -o function -lm

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

int main(int argc, char **argv)
{

  int i, j, n;

  int nnod, nele;

  int nod;
  int ele;
  int type, ntags, elemreg, physreg, partition;

  float cx;
  float cy;
  float cz;

  char descr[512];
 
  char *path;
  char *mshfilein;
  char *mshfileout;

  FILE *fpmshin;
  FILE *fpmshout;

  float *vx;
  float *vy;
  float *vz;

  int *ncor;
  int nod_malloc;

  int e[8];

  float d;
  
  printf ("\n");
  printf ("*****************************************\n");
  printf ("*                                       *\n");
  printf ("*    OpenFVM - Map function to mesh     *\n");
  printf ("*                                       *\n");
  printf ("*****************************************\n");
  printf ("\n");

  // Convert mesh
  path = calloc (strlen (argv[1]), sizeof (char));

  mshfilein = calloc (strlen (argv[1]) + 12, sizeof (char));
  mshfileout = calloc (strlen (argv[1]) + 12, sizeof (char));

  strcpy (path, argv[1]);

  sprintf (mshfilein, "%s.msh", path);
  sprintf (mshfileout, "%s_out.msh", path);

  fpmshin = fopen(mshfilein, "r");
  fpmshout = fopen(mshfileout, "w");

  do
    {

      do
	{

	  strcpy (descr, "");

	  fscanf (fpmshin, "%s", descr);
	  
	  if (strcmp (descr, "$Nodes") == 0)
	    {
	      fprintf (fpmshout, "$Nodes\n");
	      break;
	    }

	  if (strcmp (descr, "$Elements") == 0)
	    {
	      fprintf (fpmshout, "$Elements\n");
	      break;
	    }

	}
      while (!feof (fpmshin));    

      if (strcmp (descr, "$Nodes") == 0)
	{
	  
	  fscanf (fpmshin, "%d", &nnod);
	  fprintf (fpmshout, "%d\n", nnod);
	  	 
 	  nod_malloc = nnod;
	  nod_malloc++;

	  ncor = calloc (nod_malloc, sizeof (int));

	  vx = calloc (nnod, sizeof (float));
	  vy = calloc (nnod, sizeof (float));
	  vz = calloc (nnod, sizeof (float));

	  for (i = 0; i < nnod; i++)
	    {
	      	    
	      fscanf (fpmshin, "%d %f %f %f", &nod, &cx, &cy, &cz);
	      fprintf(fpmshout, "%d %f %f %f\n", nod, cx, cy, cz);
	    
  	      if (nod >= nod_malloc)
		{

		  nod_malloc *= 2;
		  nod_malloc++;
		  
		  ncor = realloc (ncor, nod_malloc * sizeof (int));
		}
	  
	      vx[i] = cx;
	      vy[i] = cy;
	      vz[i] = cz;

	      ncor[nod] = i;

	    }

	  fprintf (fpmshout, "$EndNodes\n");

	}
           
      if (strcmp (descr, "$Elements") == 0)
	{
	  
	  fscanf (fpmshin, "%d", &nele);
	  fprintf (fpmshout, "%d\n", nele);
	  	  
	  for (i = 0; i < nele; i++)
	    {
	      
	      fscanf (fpmshin, "%d %d %d %d %d %d", &ele, &type, &ntags, &physreg, &elemreg, &partition);

	      if (type == 2) nnod = 3;
	      if (type == 3) nnod = 4;
	      if (type == 4) nnod = 4;
	      if (type == 5) nnod = 8;
	      if (type == 6) nnod = 6;

	      for (j = 0; j < nnod; j++)
		{
		  fscanf (fpmshin, "%d", &nod);	
		  e[j] = nod;
		}
	      
	      cx = 0.0;
	      cy = 0.0;
	      cz = 0.0;

	      for (j = 0; j < nnod; j++)
		{
		  cx += vx[ncor[e[j]]];
		  cy += vy[ncor[e[j]]];
		  cz += vz[ncor[e[j]]];
		}
	      
	      if (nnod > 0)
		{
		  cx /= nnod;  
		  cy /= nnod;  
		  cz /= nnod;  
		}	      

              n = 0;

	      d = sqrt((cx - 0.5)*(cx - 0.5)+(cy - 0.5)*(cy - 0.5));
	      if (d < 0.25 && (cx >= 0.55 || cx <= 0.45 || cy >= 0.5)) n++;

	      for (j = 0; j < nnod; j++)
		{
		    d = sqrt((vx[ncor[e[j]]] - 0.5)*(vx[ncor[e[j]]] - 0.5)+(vy[ncor[e[j]]] - 0.5)*(vy[ncor[e[j]]] - 0.5));
		    if (d < 0.25 && (vx[ncor[e[j]]] >= 0.55 || vx[ncor[e[j]]] <= 0.45 || vy[ncor[e[j]]] >= 0.5)) n++;
                }

              physreg = 1; 

              if (n == 3)
              {
		  physreg = 2; 
              }

              if (n == 5)
              {
		  physreg = 3; 
              }

              if (n == 7)
              {
		  physreg = 4; 
              }

              if (n == 9)
              {
		  physreg = 5; 
              }
		
	      fprintf (fpmshout, "%d %d %d %d %d %d", ele, type, ntags, physreg, elemreg, partition);
	      
	      for (j = 0; j < nnod; j++)
		{
		  fprintf(fpmshout, " %d", e[j]);		      		      
		}      

	      fprintf(fpmshout, "\n");		      		     		      
	      	      
	    }

	  fprintf (fpmshout, "EndElements\n");
	  
	} 
    }  
  while (!feof (fpmshin));
  
  fclose(fpmshin);
  fclose(fpmshout);

  remove(mshfilein);
  rename(mshfileout, mshfilein);

}



